package com.humuson.amc.client.context;

import com.humuson.amc.client.AmcClient;
import com.humuson.amc.client.constant.SessionStrategyType;

public class SessionStore {
	
	private SessionStrategyType strategyType = SessionStrategyType.ThreadLocal;
	
	private SessionStrategy strategy;
	
	private AmcClient client;
	
	public SessionStore(AmcClient client) {
		initialize(client, null);
	}
	
	public SessionStore(AmcClient client ,SessionStrategyType strategyType) {
		initialize(client, strategyType);
	}
	
	public void clearSession() {
		strategy.clearSession();
	}

	public Session getSession() {
		return strategy.getSession();
	}

	public void setSession(Session session) {
		strategy.setSession(session);
	}

	public Session createEmptySession() {
		return strategy.createEmptySession();
	}
	
	private void initialize(AmcClient client, SessionStrategyType strategyType) {
		
		this.client = client;
		
		initializeStrategy(strategyType);
	}
	
	private void initializeStrategy(SessionStrategyType strategyType) {
		
		this.strategyType = strategyType;
		
		if ( strategyType == SessionStrategyType.ThreadLocal ) {
			
			strategy = new ThreadLocalSessionStrategy(client.getClientContext());
			
		} else if ( strategyType == SessionStrategyType.Global ) {
			
			strategy = new GlobalSessionStrategy();
			
		} else {
			
			strategy = new ThreadLocalSessionStrategy(client.getClientContext());
		}
	}
	
	public void setStrategy(SessionStrategyType strategyType) {
		
		if ( strategyType != null && this.strategyType != strategyType ) {
			
			initializeStrategy(strategyType);	
		}
	}
}