package com.humuson.amc.client.context;

public class ThreadLocalSessionStrategy implements SessionStrategy {
	
	private ThreadLocal<Session> sessionHolder = new ThreadLocal<Session>();
	
	ClientContext context;
	
	public ThreadLocalSessionStrategy(ClientContext context) {
		this.context = context;
	}
	
	public void clearSession() {
		sessionHolder.remove();
	}

	public Session getSession() {
		Session session = sessionHolder.get();

		if (session == null) {
			session = createEmptySession();
			sessionHolder.set(session);
		}

		return session;
	}

	public void setSession(Session session) {
		if ( session == null ) {
			throw new IllegalArgumentException("session not null");
		}
		sessionHolder.set(session);
	}

	public Session createEmptySession() {
		Session session = new Session();
		session.setAuthentication(context.getDefaultAuthentication().copy());
		return session;
	}
}
