package com.humuson.amc.client.http;

import java.lang.reflect.Type;
import java.util.Map;

import com.humuson.amc.client.AmcClient;
import com.humuson.amc.client.context.SessionStore;
import com.humuson.amc.client.model.Authentication;
import com.humuson.amc.client.model.Request;
import com.humuson.amc.client.model.Response;
import com.humuson.amc.client.util.HttpUtil;

public class NonSecurityHttpComunicator implements HttpComunicator {
	
	AmcClient client;
	
	public NonSecurityHttpComunicator(AmcClient client) {
		this.client = client;
	}
	
	@Override
	public Authentication authorize() {
		
		return client.getClientContext().getDefaultAuthentication();
	}

	@Override
	public Authentication authorize(Integer timeout) {
		return client.getClientContext().getDefaultAuthentication();
	}
		
	@Override
	public <T> Response<T> executePost(String path, Request request, Type type) {
		
		Map<String,Object> params = null;
		Integer timeout = null;
		Integer retry = null;
		
		if ( request != null ) {
			params	= request.getParams();
			timeout	= request.getTimeout();
			retry = request.getRetryCount();
		}
		
		return executePost(path, request.getHttpHeaders(), params, timeout, type, retry);
	}
	
	@Override
	public <T> Response<T> executePost(
			String path, Map<String,String> headers, 
			Map<String,Object> params, Integer timeout, 
			Type type, Integer retry) {
				
		String uri = client.makeFullPath(path);
				
		Response<T> response =  HttpUtil.executePost(uri, headers, params, timeout, type, retry);
		
		return response;
	}

	@Override
	public <T> Response<T> executeGet(String path, Request request, Type type) {
		
		Map<String,Object> params = null;
		Integer timeout = null;
		Integer retry = null;
		
		if ( request != null ) {
			params	= request.getParams();
			timeout	= request.getTimeout();
			retry = request.getRetryCount();
		}
		
		return executeGet(path, request.getHttpHeaders(), params, timeout, type, retry);
	}

	@Override
	public <T> Response<T> executeGet(
			String path, Map<String, String> headers, 
			Map<String, Object> params, Integer timeout,
			Type type, Integer retry) {
		
		String uri = client.makeFullPath(path);
		
		Response<T> response =  HttpUtil.executeGet(uri, headers, params, timeout, type, retry);
		
		return response;
	}

	@Override
	public <T> Response<T> executePut(String path, Request request, Type type) {
		
		Map<String,Object> params = null;
		Integer timeout = null;
		Integer retry = null;
		
		if ( request != null ) {
			params	= request.getParams();
			timeout	= request.getTimeout();
			retry = request.getRetryCount();
		}
		
		return executePut(path, request.getHttpHeaders(), params, timeout, type, retry);
	}

	@Override
	public <T> Response<T> executePut(
			String path, Map<String, String> headers,
			Map<String, Object> params, Integer timeout,
			Type type, Integer retry) {
		
		String uri = client.makeFullPath(path);
		
		Response<T> response =  HttpUtil.executePut(uri, headers, params, timeout, type, retry);
		
		return response;
	}

	@Override
	public <T> Response<T> executeDelete(String path, Request request, Type type) {
		
		Map<String,Object> params = null;
		Integer timeout = null;
		Integer retry = null;
		
		if ( request != null ) {
			params	= request.getParams();
			timeout	= request.getTimeout();
			retry = request.getRetryCount();
		}
		
		return executeDelete(path, request.getHttpHeaders(), params, timeout, type, retry);
	}

	@Override
	public <T> Response<T> executeDelete(
			String path, Map<String, String> headers,
			Map<String, Object> params, Integer timeout,
			Type type, Integer retry) {
		
		String uri = client.makeFullPath(path);
		
		Response<T> response =  HttpUtil.executeDelete(uri, headers, params, timeout, type, retry);
		
		return response;
	}
	
}