package com.humuson.amc.client.model;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.codec.binary.Base64;

import com.humuson.amc.client.constant.GrantType;
import com.humuson.amc.client.exception.InvalidAuthenticationParameter;
import com.humuson.amc.client.util.ReflectionUtil;
import com.humuson.amc.client.util.StringUtils;
import com.humuson.amc.client.util.TokenUtil;


public class Authentication {
	
	private String userId;
	private String userPassword;
	private String serviceId;
	private String servicePassword;
	private String token;
	private String refreshToken;
	private GrantType grantType;
	
	public String getUserId() {
		return userId;
	}

	public Authentication setUserId(String userId) {
		this.userId = userId;
		return this;
	}

	public String getUserPassword() {
		return userPassword;
	}

	public Authentication setUserPassword(String userPassword) {
		this.userPassword = userPassword;
		return this;
	}

	public String getServiceId() {
		return serviceId;
	}

	public Authentication setServiceId(String serviceId) {
		this.serviceId = serviceId;
		return this;
	}

	public String getServicePassword() {
		return servicePassword;
	}

	public Authentication setServicePassword(String servicePassword) {
		this.servicePassword = servicePassword;
		return this;
	}

	public String getToken() {
		return token;
	}

	public Authentication setToken(String token) {
		this.token = token;
		return this;
	}

	public String getRefreshToken() {
		return refreshToken;
	}

	public Authentication setRefreshToken(String refreshToken) {
		this.refreshToken = refreshToken;
		return this;
	}
	
	public Map<String,String> getAuthorizedTokenHeader() {
		Map<String,String> header = new HashMap<String,String>();
		header.put("Authorization", "Bearer "+token);
		return header;
	}
	
	
	public GrantType getGrantType() {
		return grantType;
	}

	public Authentication setGrantType(GrantType grantType) {
		this.grantType = grantType;
		return this;
	}

	public Map<String,String> getClientAuthorizationHeader() {
		
		Map<String,String> header = new HashMap<String,String>();
		
		String value = serviceId+":"+servicePassword;
		byte[] valueBytes;
		
		try {
		
			valueBytes = value.getBytes("UTF-8");
			
			String encoded = Base64.encodeBase64String(valueBytes);
			
			header.put("Authorization", "Basic "+encoded);
			
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		
		return header;
	}
	
	public boolean isValidToken() {
				
		if ( TokenUtil.isValidToken(token) ) {
			return true;
		}
				
		return false;
	}
		
	public boolean isAuthenticated() {
		
		if ( isValidToken() ) {
			return true;
		}
		 
		return false;
	}

	public GrantType findRequestTokenGrantType() {
		
		if ( StringUtils.isEmpty(serviceId) || StringUtils.isEmpty(servicePassword) ) {
			throw new InvalidAuthenticationParameter("service id and service password not found");
		}
		
		if ( grantType != null ) {
			return grantType;
		}
		
		if ( StringUtils.isNotEmpty(userId) ) {
			
			if ( StringUtils.isNotEmpty(userPassword) ) {
				return GrantType.Password;
			} else {
				return GrantType.ClientUserCredentials;
			}
			
		} else if ( StringUtils.isNotEmpty(refreshToken) ) {
			
			return GrantType.RefreshToken;
			
		} else {
			
			return GrantType.ClientCredentials;
		}
		
	}

	@Override
	public String toString() {
		return ReflectionUtil.toJson(this);
	}
	
	public Authentication copy() {
		Authentication copyAuth = new Authentication();
		copyAuth.setUserId(userId);
		copyAuth.setUserPassword(userPassword);
		copyAuth.setServiceId(serviceId);
		copyAuth.setServicePassword(servicePassword);
		copyAuth.setGrantType(grantType);
		copyAuth.setToken(token);
		copyAuth.setRefreshToken(refreshToken);
		return copyAuth;
	}
	
	public boolean isSameWithoutToken(Authentication auth) {
		
		if ( auth == null ) {
			return false;
		}
		
		if ( !StringUtils.isEqual(userId, auth.getUserId()) ) {
			return false;
		}
		
		if ( !StringUtils.isEqual(userPassword, auth.getUserPassword()) ) {
			return false;
		}
		
		if ( !StringUtils.isEqual(serviceId, auth.getServiceId()) ) {
			return false;
		}
		
		if ( !StringUtils.isEqual(servicePassword, auth.getServicePassword()) ) {
			return false;
		}
		
		GrantType gt = auth.getGrantType();
		if ( grantType != null || gt != null ) {
			if ( ( grantType != null && gt == null ) || ( grantType == null && gt != null ) ) {
				return false;
			} else if ( !StringUtils.isEqual(grantType.getCode(), auth.getGrantType().getCode()) ) {
				return false;
			}
		}
				
		return true;
	}
	
}
