package com.humuson.amc.client.model;

import java.util.HashMap;
import java.util.Map;

import com.humuson.amc.client.util.ReflectionUtil;

public abstract class Request {
	
	Integer timeout;
	
	Integer retryCount;
	
	Map<String,Object> extraParams;
	
	Map<String,String> headers;

	public Integer getTimeout() {
		return timeout;
	}

	public void setTimeout(Integer timeout) {
		this.timeout = timeout;
	}

	public Integer getRetryCount() {
		return retryCount;
	}

	public void setRetryCount(Integer retryCount) {
		this.retryCount = retryCount;
	}

	public Map<String, Object> getExtraParams() {
		return extraParams;
	}

	public void setExtraParams(Map<String, Object> extraParams) {
		this.extraParams = extraParams;
	}
	
	public void addParam(String key, Object value) {
		if ( key == null || "".equals(key) || value == null ) {
			return;
		}
		
		if ( extraParams == null ) {
			extraParams = new HashMap<String, Object>();
		}
		
		extraParams.put(key, value);
	}
	
	public void validate() {}
	
	protected abstract Map<String, Object> toParams();
	
	public Map<String, Object> getParams() {
		
		validate();
		
		Map<String, Object> params = null;
		
		if ( extraParams == null ) {
			params = new HashMap<String,Object>();
		} else {
			params = extraParams;
		}
		
		Map<String, Object> p = toParams();
		
		if ( p != null && p != extraParams ) {
			params.putAll(p);
		}
		
		return params;
	}
	
	public void setHttpHeader(String key, String value) {
		
		if ( headers == null ) {
			headers = new HashMap<String,String>();
		}
		
		headers.put(key, value);
	}
	
	public Map<String, String> getHttpHeaders() {
		
		if ( isJsonContentType() ) {
			setHttpHeader("Content-Type", "application/json");
		}
		
		return headers;
	}
	
	protected abstract boolean isJsonContentType();
	
	@Override
	public String toString() {
		return String.format("Request[headers:%s, params:%s, timeout:%s]", ReflectionUtil.toJson(getHttpHeaders()), ReflectionUtil.toJson(getParams()), getTimeout());
	}
}
