package com.humuson.amc.client.model.request;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.humuson.amc.client.constant.EncryptType;
import com.humuson.amc.client.exception.NoSuchException;
import com.humuson.amc.client.model.Request;
import com.humuson.amc.client.model.response.Contact;
import com.humuson.amc.client.util.ContactSecurityUtil;
import com.humuson.amc.client.util.ValidateUtil;

public class ContactRegistRequest extends Request {
	
	String siteKey;
	
	List<Contact> contacts;
	
	//index생성시 seq를 쓸지 말지 결정함
	boolean useSiteIndex = true;
	
	boolean syncContact = false;
	
	EncryptType encryptType = EncryptType.SHA;
	
	public String getSiteKey() {
		return siteKey;
	}

	public void setSiteKey(String siteKey) {
		this.siteKey = siteKey;
	}

	public List<Contact> getContacts() {
		return contacts;
	}

	public void setContacts(List<Contact> contacts) {
		this.contacts = contacts;
	}

	public boolean isUseSiteIndex() {
		return useSiteIndex;
	}

	public void setUseSiteIndex(boolean useSiteIndex) {
		this.useSiteIndex = useSiteIndex;
	}

	public boolean isSyncContact() {
		return syncContact;
	}

	public void setSyncContact(boolean syncContact) {
		this.syncContact = syncContact;
	}
	
	public void setEncryptType(EncryptType encryptType) {
		
		if ( encryptType == EncryptType.HE_APP ) {
			throw new NoSuchException(encryptType+" is not available this request");
		}
		
		if ( encryptType == null ) {
			this.encryptType = EncryptType.NONE;
		} else {
			this.encryptType = encryptType;
		}
	}
	
	public EncryptType getEncryptType() {
		return encryptType;
	}

	@Override
	public void validate() {
		ValidateUtil.checkEmptyAll(
				"siteKey", siteKey,
				"contacts", contacts);
	}

	@Override
	protected Map<String, Object> toParams() {
		
		Map<String, Object> params = new HashMap<String, Object>();
		
		if ( EncryptType.NONE != encryptType ) {
			contacts = ContactSecurityUtil.encode(contacts, encryptType, siteKey);	
		}
		
		params.put("siteKey", siteKey);
		params.put("contacts", contacts);
		params.put("useSiteIndex", useSiteIndex);
		params.put("syncContact", syncContact);
		params.put("enc", encryptType.getType());
		
		return params;
	}

	@Override
	protected boolean isJsonContentType() {
		return true;
	}
}