package com.humuson.amc.client.model.request;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.humuson.amc.client.constant.ContentType;
import com.humuson.amc.client.info.Required;
import com.humuson.amc.client.model.Request;
import com.humuson.amc.client.util.ValidateUtil;

public class ContentRequest extends Request {
	
	@Required(or={"content"})
	private String url;
	
	@Required(or={"url"})
	private String content;
	
	private String title;
	
	private String msgKey;
	
	private ContentType contentType;
	
	private Set<String> channelKeys = new HashSet<String>();
	
	private List<String> tags = new ArrayList<String>();
	
	private String encoding;
	
	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}
	
	public String getContent() {
		return content;
	}

	public void setContent(String content) {
		this.content = content;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}
	
	public String getMsgKey() {
		return msgKey;
	}

	public void setMsgKey(String msgKey) {
		this.msgKey = msgKey;
	}
	
	public ContentType getContentType() {
		return contentType;
	}

	public void setContentType(ContentType type) {
		this.contentType = type;
	}

	public void setChannelKeys(Set<String> channelKeys) {
		this.channelKeys = channelKeys;
	}

	public Set<String> getChannelKeys() {
		return channelKeys;
	}
	
	public String getEncoding() {
		return encoding;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	public void addChannelKey(String channelKey) {
		if ( channelKeys == null ) {
			channelKeys = new HashSet<String>();
		}
		channelKeys.add(channelKey);
	}
	
	public void removeChannelKey(String channelKey) {
		channelKeys.remove(channelKey);
	}
	
	public void resetChannelKeys() {
		if ( channelKeys != null ) channelKeys.clear();
	}
	
	public List<String> getTags() {
		return tags;
	}

	public void setTags(List<String> tags) {
		this.tags = tags;
	}
	
	public void resetTags() {
		if ( tags != null ) tags.clear();
	}
	
	public void addTag(String tag) {
		if ( tags == null ) {
			tags = new ArrayList<String>();
		}
		tags.add(tag);
	}
	
	public void removeTag(String tag) {
		tags.remove(tag);
	}

	@Override
	public void validate() {
				
		ValidateUtil.checkEmptyIn(
				"contentType", contentType,
				"msgKey", msgKey);
		
		ValidateUtil.checkEmptyAll(
				"url", url,
				"content", content);
	}

	@Override
	protected Map<String, Object> toParams() {
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("url", url);
		params.put("content", content);
		params.put("title", title);
		params.put("msgKey", msgKey);
		params.put("type", this.contentType.getCode());
		params.put("channelKeys", channelKeys);
		params.put("tags", tags);
		params.put("encoding", encoding);
		
		return params;
	}

	@Override
	protected boolean isJsonContentType() {
		return false;
	}
}
