package com.humuson.amc.client.model.request;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.humuson.amc.client.constant.GrantType;
import com.humuson.amc.client.exception.InvalidParameterException;
import com.humuson.amc.client.model.Request;
import com.humuson.amc.client.util.ValidateUtil;


public class ServiceRegistRequest extends Request{
	
	String id;
	
	String password;

	String name;

	List<GrantType>  grantTypes;
	
	String redirectUri;
		
	boolean autoApprove;
	
	Integer accessTokenValiditySeconds = 86400;
	
	Integer refreshTokenValiditySeconds = 172800;

	
	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public List<GrantType> getGrantTypes() {
		return grantTypes;
	}

	public void addGrantTypes(GrantType grantType) {
		if ( grantTypes == null ) grantTypes = new ArrayList<GrantType>();
		grantTypes.add(grantType);
	}
	
	public void setGrantTypes(List<GrantType> grantTypes) {
		this.grantTypes = grantTypes;
	}

	public String getRedirectUri() {
		return redirectUri;
	}

	public void setRedirectUri(String redirectUri) {
		this.redirectUri = redirectUri;
	}

	public boolean isAutoApprove() {
		return autoApprove;
	}

	public void setAutoApprove(boolean autoApprove) {
		this.autoApprove = autoApprove;
	}

	public Integer getAccessTokenValiditySeconds() {
		return accessTokenValiditySeconds;
	}

	public void setAccessTokenValiditySeconds(Integer accessTokenValiditySeconds) {
		this.accessTokenValiditySeconds = accessTokenValiditySeconds;
	}

	public Integer getRefreshTokenValiditySeconds() {
		return refreshTokenValiditySeconds;
	}

	public void setRefreshTokenValiditySeconds(Integer refreshTokenValiditySeconds) {
		this.refreshTokenValiditySeconds = refreshTokenValiditySeconds;
	}

	@Override
	public void validate() {
		
		ValidateUtil.checkEmptyIn(
				"id", id,
				"name", name,
				"password", password,
				"grantTypes", grantTypes);
	}
	
	@Override
	protected Map<String, Object> toParams() {
		
		Map<String, Object> params = new HashMap<String, Object>();
		
		//기본 role 세팅
		List<String> roles = new ArrayList<String>();
		roles.add("SERVICE");
		params.put("roles", roles);
		
		//기본 scope 세팅
		List<String> scopes = new ArrayList<String>();
		scopes.add("READ");
		params.put("scopes", scopes);
		
		
		List<String> grantTypeList = new ArrayList<String>();
		for ( GrantType grantType : grantTypes ) {
			grantTypeList.add(grantType.getCode());
		}
		params.put("grantTypes", grantTypeList);
		
		params.put("id", id);
		params.put("password", password);
		params.put("name", name);
		params.put("redirectUri", redirectUri);
		params.put("accessTokenValiditySeconds", accessTokenValiditySeconds);
		params.put("refreshTokenValiditySeconds", refreshTokenValiditySeconds);
		params.put("autoApprove", autoApprove);
		
		return params;
		
	}

	@Override
	protected boolean isJsonContentType() {
		return true;
	}
	
	
}
