package com.humuson.amc.client.model.request;

import java.util.HashMap;
import java.util.Map;

import com.humuson.amc.client.exception.InvalidParameterException;
import com.humuson.amc.client.model.Request;
import com.humuson.amc.client.util.ValidateUtil;

public class SharingCostRequest extends Request {
	
	String siteKey;
	
	private Map<Long,Integer> sharingCostDeductions = new HashMap<Long,Integer>();
	private Map<String,Map<Long,Integer>> sharingFailCount = new HashMap<String,Map<Long,Integer>>();
	
	public String getSiteKey() {
		return siteKey;
	}

	public void setSiteKey(String siteKey) {
		this.siteKey = siteKey;
	}
	
	/**
	 * seq에 해당하는 sharing 비용을 차감 혹은 취소한다
	 * 
	 * 건수(cnt) 값이 양수라면 차감하고 음수라면 취소한다
	 * 
	 * @param seq
	 * @param cnt
	 */
	public void setSharingCostDeductCnt(Long seq, int cnt) {
		sharingCostDeductions.put(seq, cnt);
	}
	
	public void setSharingCostDeductions(Map<Long, Integer> sharingCostDeductions) {
		this.sharingCostDeductions = sharingCostDeductions;
	}

//	public void setSharingFailCount(Map<String, List<Sharing>> sharingFailCount) {
//		this.sharingFailCount = sharingFailCount;
//	}
	
	public void setSharingFailCount(Map<String,Map<Long,Integer>> sharingFailCount) {
		this.sharingFailCount = sharingFailCount;
	}
	
	@Override
	public void validate() {
		ValidateUtil.checkEmptyIn("siteKey", siteKey);
		if ( sharingCostDeductions == null && sharingCostDeductions.size() == 0
				&& sharingFailCount == null && sharingFailCount.size() == 0) {
			throw new InvalidParameterException("sharing count is empty. check sharing.cost.deductions or sharing.fail.count");
		}
	}
	
	@Override
	protected Map<String, Object> toParams() {
		
		Map<String, Object> params = new HashMap<String, Object>();
		
		params.put("siteKey", siteKey);
		params.put("sharingCostDeductions", sharingCostDeductions);
		params.put("sharingFailCount", sharingFailCount);
				
		return params;
	}

	@Override
	protected boolean isJsonContentType() {
		return true;
	}
	
	/*
	public static void main(String[] args) {
		SharingCostRequest request = new SharingCostRequest();
		request.setSiteKey("asdf1234");
		request.setSharingCostDeductions(null);
		
		Map<String,Map<Long,Integer>> map = new HashMap<String,Map<Long,Integer>>();
		Map<Long,Integer> paramMap = new HashMap<Long,Integer>();
		paramMap.put(1l, 1);
		paramMap.put(2l, 2);
		paramMap.put(3l, 3);
		
		map.put("123456", paramMap);
		
		request.setSharingFailCount(map);
		
		Gson gson = new Gson();
		System.out.println(gson.toJson(request));
	}
	*/
}