package com.humuson.amc.client.model.request;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import com.humuson.amc.client.constant.Action;
import com.humuson.amc.client.constant.ChannelType;
import com.humuson.amc.client.constant.Platform;
import com.humuson.amc.client.constant.TargetType;
import com.humuson.amc.client.exception.InvalidParameterException;
import com.humuson.amc.client.util.StringUtils;
import com.humuson.amc.client.util.ValidateUtil;

public class TargetForm {
	
	public static final Pattern DATE_STRING_PATTERN = Pattern.compile("[1-9][0-9]{3}-[0-1][0-9]-[0-3][0-9]");
	
	TargetType type;
	
	Set<String> tags;
	
	Set<Action> actions;
	
	Long audienceId;
	
	Set<Platform> platforms;
	
	Set<ChannelType> channelTypes;
	
	Set<String> appKeys;
	
	Set<String> msgKeys;
	
	String startDate;
	
	String endDate;
	
	boolean reverseTarget = false;

	boolean excludeMsgReact = false;
	
	public TargetType getType() {
		return type;
	}

	public void setType(TargetType type) {
		this.type = type;
	}

	public Set<String> getTags() {
		return tags;
	}
	
	public void setTags(Set<String> tags) {
		
		if ( tags != null ) {
			
			Set<String> _tags = new HashSet<String>();
			
			String _tag = null;
			for ( String tag : tags ) {
				
				_tag = StringUtils.toLowerCase(tag);
				
				if ( StringUtils.isNotEmpty(_tag) ) {
					_tags.add(_tag);	
				}
			}
			this.tags = _tags;
		}
	}
	
	public void addTag(String tag) {
		
		if ( tags == null ) {
			tags = new HashSet<String>();
		}
		
		String _tag = StringUtils.toLowerCase(tag);
		
		if ( StringUtils.isNotEmpty(_tag) ) {
			tags.add(_tag);	
		}
	}

	public Set<Action> getActions() {
		return actions;
	}
	
	public void addAction(Action action) {
		if ( actions == null ) {
			actions = new HashSet<Action>();
		}
		if ( action != null ) {
			actions.add(action);
		}
	}

	public void setActions(Set<Action> actions) {
		this.actions = actions;
	}

	public Long getAudienceId() {
		return audienceId;
	}

	public void setAudienceId(Long audienceId) {
		this.audienceId = audienceId;
	}

	public Set<Platform> getPlatforms() {
		return platforms;
	}

	public void setPlatforms(Set<Platform> platforms) {
		this.platforms = platforms;
	}
	
	public void addPlatform(Platform platform) {
		if ( platforms == null ) {
			platforms = new HashSet<Platform>();
		}
		if ( platforms != null ) {
			platforms.add(platform);	
		}
	}

	public boolean isReverseTarget() {
		return reverseTarget;
	}

	public void setReverseTarget(boolean reverseTarget) {
		this.reverseTarget = reverseTarget;
	}

	public Set<ChannelType> getChannelTypes() {
		return channelTypes;
	}

	public void setChannelTypes(Set<ChannelType> channelTypes) {
		this.channelTypes = channelTypes;
	}
	
	public void addChannelType(ChannelType channelType) {
		if ( channelTypes == null ) channelTypes = new HashSet<ChannelType>();
		if ( channelType != null ) {
			channelTypes.add(channelType);	
		}
	}

	public Set<String> getAppKeys() {
		return appKeys;
	}
	
	public void addAppKey(String appKey) {
		if ( appKeys == null ) appKeys = new HashSet<String>();
		if ( StringUtils.isNotEmpty(appKey) ) {
			appKeys.add(appKey);	
		}
	}

	public void setAppKeys(Set<String> appKeys) {
		this.appKeys = appKeys;
	}

	public Set<String> getMsgKeys() {
		return msgKeys;
	}
	
	public void addMsgKey(String msgKey) {
		if ( msgKeys == null ) msgKeys = new HashSet<String>();
		if ( StringUtils.isNotEmpty(msgKey) ) {
			msgKeys.add(msgKey);	
		}
	}

	public void setMsgKeys(Set<String> msgKeys) {
		this.msgKeys = msgKeys;
	}

	public String getStartDate() {
		return startDate;
	}

	public void setStartDate(String startDate) {
		if ( !DATE_STRING_PATTERN.matcher(startDate).matches() ) {
			throw new InvalidParameterException("startDate format is yyyy-MM-dd");
		}
		this.startDate = startDate;
	}

	public String getEndDate() {
		return endDate;
	}

	public void setEndDate(String endDate) {
		if ( !DATE_STRING_PATTERN.matcher(endDate).matches() ) {
			throw new InvalidParameterException("endDate format is yyyy-MM-dd");
		}
		this.endDate = endDate;
	}
	
	public boolean isExcludeMsgReact() {
		return excludeMsgReact;
	}

	public void setExcludeMsgReact(boolean excludeMsgReact) {
		this.excludeMsgReact = excludeMsgReact;
	}

	public void validate() {
		ValidateUtil.checkEmpty("type", type);
	}
	
	public Map<String,Object> toRequestMap() {
		
		Map<String,Object> result = new HashMap<String,Object>();
		
		result.put("type", type.getCode());
		result.put("reverseTarget", reverseTarget);
		result.put("excludeMsgReact", excludeMsgReact);
		
		if ( tags != null && tags.size() > 0 ) {
			result.put("tags", tags);
		}
		
		if ( actions != null && actions.size() > 0 ) {
			List<String> _actions = new ArrayList<String>(actions.size());
			for ( Action action : actions ) {
				_actions.add(action.getName());
			}
			result.put("actions", _actions);
		}
		
		if ( audienceId != null ) {
			result.put("audienceId", audienceId);
		}
		
		if ( platforms != null && platforms.size() > 0 ) {
			List<String> _platforms = new ArrayList<String>(platforms.size());
			for ( Platform platform : platforms ) {
				_platforms.add(platform.getCode());
			}
			result.put("platforms", _platforms);
		}
		
		if ( channelTypes != null && channelTypes.size() > 0 ) {
			List<String> _channelTypes = new ArrayList<String>(channelTypes.size());
			for ( ChannelType channelType : channelTypes ) {
				_channelTypes.add(channelType.getCode());
			}
			result.put("channelTypes", _channelTypes);
		}
		
		if ( appKeys != null && appKeys.size() > 0 ) {
			result.put("appKeys", appKeys);
		}
		
		if ( msgKeys != null && msgKeys.size() > 0 ) {
			result.put("msgKeys", msgKeys);
		}
		
		if ( StringUtils.isNotEmpty(startDate) ) {
			result.put("startDate", startDate);
		}
		
		if ( StringUtils.isNotEmpty(endDate) ) {
			result.put("endDate", endDate);
		}
		
		return result;
	}
	
}
