package com.humuson.amc.client.util;

import java.util.List;

import com.humuson.amc.client.constant.EncryptType;
import com.humuson.amc.client.exception.InvalidParameterException;
import com.humuson.amc.client.model.response.Contact;
import com.humuson.amc.client.security.Decryptor;
import com.humuson.amc.client.security.Encryptor;

public class ContactSecurityUtil {
	
	public static String encode(String str, EncryptType encType, String key) {
		
		if ( str == null || encType == null ) return null;
		
		switch(encType) {
			case HE_SITE:
			case HE_APP:
				if ( key == null ) throw new InvalidParameterException(String.format("encrypt key not found. encType:%s,key:%s",encType.getType(),key));
				return Encryptor.encrypt(str, key);
			case SHA:
				return Encryptor.encryptSHA(str);
			default:
				return str;
		}
	}
	
	public static String decode(String str, EncryptType encType, String key) {
		
		if ( str == null || encType == null ) return null;
		
		switch(encType) {
			case HE_SITE:
			case HE_APP:
				if ( key == null ) throw new InvalidParameterException(String.format("decrypt key not found. encType:%s,key:%s",encType.getType(),key));
				return Decryptor.decrypt(str, key);
			default:
				return str;
		}
	}
	
	public static Contact encode(Contact contact, EncryptType encType, String key) {
		if ( contact == null ) return null;
		contact.setEmail(encode(contact.getEmail(), encType, key));
		contact.setPhone(encode(contact.getPhone(), encType, key));
		contact.setData1(encode(contact.getData1(), encType, key));
		contact.setData2(encode(contact.getData2(), encType, key));
		contact.setData3(encode(contact.getData3(), encType, key));
		contact.setData4(encode(contact.getData4(), encType, key));
		contact.setData5(encode(contact.getData5(), encType, key));
		return contact;
	}
	
	public static Contact decode(Contact contact, EncryptType encType, String key) {
		if ( contact == null ) return null;
		contact.setEmail(decode(contact.getEmail(), encType, key));
		contact.setPhone(decode(contact.getPhone(), encType, key));
		contact.setData1(decode(contact.getData1(), encType, key));
		contact.setData2(decode(contact.getData2(), encType, key));
		contact.setData3(decode(contact.getData3(), encType, key));
		contact.setData4(decode(contact.getData4(), encType, key));
		contact.setData5(decode(contact.getData5(), encType, key));
		return contact;
	}
	
	public static List<Contact> encode(List<Contact> contacts, EncryptType encType, String key) {
		if ( contacts == null || contacts.isEmpty() ) return null;
		for ( Contact contact : contacts ) {
			encode(contact, encType, key);
		}
		return contacts;
	}
	
	public static List<Contact> decode(List<Contact> contacts, EncryptType encType, String key) {
		if ( contacts == null || contacts.isEmpty() ) return null;
		for ( Contact contact : contacts ) {
			decode(contact, encType, key);
		}
		return contacts;
	}
}
